"use strict";

const util_1 = require("./util/util");
const fs_extra_p_1 = require("fs-extra-p");
const path = require("path");
const os_1 = require("os");
const binDownload_1 = require("./util/binDownload");
//noinspection JSUnusedLocalSymbols
const __awaiter = require("./util/awaiter");
const TOOLS_VERSION = "winCodeSign-1.0.0";
function sign(options) {
    return __awaiter(this, void 0, void 0, function* () {
        let hashes = options.hash;
        if (hashes == null) {
            hashes = ["sha1", "sha256"];
        } else {
            hashes = Array.isArray(hashes) ? hashes.slice() : [hashes];
        }
        const isWin = process.platform === "win32";
        let nest = false;
        //noinspection JSUnusedAssignment
        let outputPath = "";
        for (let hash of hashes) {
            outputPath = isWin ? options.path : getOutputPath(options.path, hash);
            yield spawnSign(options, options.path, outputPath, hash, nest);
            nest = true;
            if (!isWin) {
                yield fs_extra_p_1.rename(outputPath, options.path);
            }
        }
    });
}
exports.sign = sign;
// on windows be aware of http://stackoverflow.com/a/32640183/1910191
function spawnSign(options, inputPath, outputPath, hash, nest) {
    return __awaiter(this, void 0, void 0, function* () {
        const timestampingServiceUrl = "http://timestamp.verisign.com/scripts/timstamp.dll";
        const isWin = process.platform === "win32";
        const args = isWin ? ["sign", nest || hash === "sha256" ? "/tr" : "/t", nest || hash === "sha256" ? "http://timestamp.comodoca.com/rfc3161" : timestampingServiceUrl] : ["-in", inputPath, "-out", outputPath, "-t", timestampingServiceUrl];
        const certExtension = path.extname(options.cert);
        if (certExtension === ".p12" || certExtension === ".pfx") {
            args.push(isWin ? "/f" : "-pkcs12", options.cert);
        } else {
            args.push(isWin ? "/f" : "-certs", options.cert);
            // todo win maybe incorrect
            args.push(isWin ? "/csp" : "-key", options.key);
        }
        if (!isWin || hash !== "sha1") {
            args.push(isWin ? "/fd" : "-h", hash);
            if (isWin) {
                args.push("/td", "sha256");
            }
        }
        if (options.name) {
            args.push(isWin ? "/d" : "-n", options.name);
        }
        if (options.site) {
            args.push(isWin ? "/du" : "-i", options.site);
        }
        if (nest) {
            args.push(isWin ? "/as" : "-nest");
        }
        if (options.password) {
            args.push(isWin ? "/p" : "-pass", options.password);
        }
        if (options.passwordPath) {
            if (isWin) {
                throw new Error("-readpass is not supported on Windows");
            }
            args.push("-readpass", options.passwordPath);
        }
        if (isWin) {
            // must be last argument
            args.push(inputPath);
        }
        return yield util_1.spawn((yield getToolPath(options)), args);
    });
}
// async function verify(options: any) {
//   const out = await exec(await getToolPath(options), [
//     "verify",
//     "-in", options.path,
//     "-require-leaf-hash", options.hash
//   ])
//   if (out.includes("No signature found.")) {
//     throw new Error("No signature found")
//   }
//   else if (out.includes("Leaf hash match: failed")) {
//     throw new Error("Leaf hash match failed")
//   }
// }
function getOutputPath(inputPath, hash) {
    const extension = path.extname(inputPath);
    return path.join(path.dirname(inputPath), `${ path.basename(inputPath, extension) }-signed-${ hash }${ extension }`);
}
function getToolPath(options) {
    return __awaiter(this, void 0, void 0, function* () {
        let result = options.signcodePath || process.env.SIGNTOOL_PATH;
        if (result) {
            return result;
        }
        if (process.env.USE_SYSTEM_SIGNCODE || process.platform === "linux") {
            return "osslsigncode";
        }
        const vendorPath = yield binDownload_1.getBin("winCodeSign", TOOLS_VERSION, `https://dl.bintray.com/electron-userland/bin/${ TOOLS_VERSION }.7z`, "0e524943dd6f03a9cee4cdaaa235e27d4945a1c9dc80ccee8ff1219e7b73ad88");
        if (process.platform === "win32") {
            return path.join(vendorPath, `windows-${ os_1.release().startsWith("6.") ? "6" : "10" }`, "signtool.exe");
        } else {
            return path.join(vendorPath, process.platform, "osslsigncode");
        }
    });
}
//# sourceMappingURL=windowsCodeSign.js.map